package com.phm.manage.service.impl;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import com.phm.manage.enums.DataTypeEnum;
import com.phm.manage.service.IFaultCaseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.phm.common.config.PHMConfig;
import com.phm.common.constant.Constants;
import com.phm.common.exception.GlobalException;
import com.phm.common.utils.file.FileUploadUtils;
import com.phm.common.utils.ip.IpUtils;
import com.phm.common.utils.uuid.Seq;
import com.phm.manage.domain.ModelInfo;
import com.phm.manage.domain.SortieParameter;
import com.phm.manage.domain.dto.ModelRequest;
import com.phm.manage.mapper.ModelInfoMapper;
import com.phm.manage.service.IModelService;
import com.phm.manage.service.ISortieParameterService;
import com.phm.manage.util.CsvUtils;
import com.phm.manage.util.FileUtil;
import com.phm.manage.util.HttpClientUtils;
import com.phm.manage.util.SnowFlakeIdGenerator;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 深度隔离模型Service业务层处理
 *
 * @author phm
 * @date 2023-08-22
 */
@Service
@Slf4j
public class ModelServiceImpl implements IModelService {

    @Autowired
    ModelInfoMapper modelInfoMapper;

    @Autowired
    private SnowFlakeIdGenerator snowFlakeIdGenerator;

    @Autowired
    private ISortieParameterService sortieParameterService;

    @Autowired
    private IFaultCaseService faultCaseService;

    @Override
    public ModelInfo selectModelById(Long id) {
        return modelInfoMapper.selectModelById(id);
    }

    @Override
    public List<ModelInfo> selectModelInfoList(ModelInfo modelInfo) {
        return modelInfoMapper.selectModelList(modelInfo);
    }

    @Override
    public int insertModelInfo(ModelInfo modelInfo) {
        modelInfo.setId(snowFlakeIdGenerator.nextId());
        return modelInfoMapper.insertModel(modelInfo);
    }

    @Override
    public int updateModelInfo(ModelInfo modelInfo) {
        return modelInfoMapper.updateModel(modelInfo);
    }

    @Override
    public int deleteModelInfoByIds(Long[] ids) {
        return modelInfoMapper.deleteModelByIds(ids);
    }

    @Override
    public int deleteModelInfoById(Long id) {
        return modelInfoMapper.deleteModelById(id);
    }

    @Override
    public String executeModel(ModelRequest modelRequest) {

        // 查询对应模型参数
        ModelInfo modelInfo = modelInfoMapper.selectModelById(modelRequest.getModelId());
        if (ObjectUtil.isEmpty(modelInfo)) {
            throw new GlobalException("模型参数信息为空，请检查！");
        }
        // TODO 模型参数
        Map<String, String> parmsMap = getParmsMap(modelInfo);
        // 数据
        SortieParameter sortieParameter = sortieParameterService.selectSortieParameterById(modelRequest.getDataId());
        if (ObjectUtil.isEmpty(sortieParameter)) {
            throw new GlobalException("选择的架次数据为空，请检查！");
        }
        String url;
        try {
            url = getFileUrl(modelInfo.getDataParms(), sortieParameter.getDataPath());
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new GlobalException("上传文件失败");
        }
        Map<String, String> parms = new HashMap<>();
        parms.put("data_url", url);
        String result = HttpClientUtils.postJson(modelInfo.getUrl(), parms);
        JSONObject jsonObject = JSONUtil.parseObj(result);
        String jsonString = jsonObject.getStr("data");
        // 故障深度隔离无结果。需要匹配故障案例
        if (Objects.equals(modelInfo.getType(), DataTypeEnum.DEEP_ISOLATION.getType())) {
            // TODO 故障案例 传入故障要素
            faultCaseService.findFaultCaseList(jsonString);
        }
        // CSV文件路径
        String csvFilePath =
                PHMConfig.getCsvFilePath() + CsvUtils.RESULT_DATA_PATH + CsvUtils.extractFilename("model_result");
        CsvUtils.jsonToFileCsv(jsonString, csvFilePath);
        SortieParameter parameter = new SortieParameter();
        parameter.setDataPath(csvFilePath).setSortieNo(sortieParameter.getSortieNo())
            .setBatchNo(IdUtil.getSnowflakeNextIdStr()).setType(modelInfo.getType());
        sortieParameterService.insertSortieParameter(parameter);
        return jsonString;
    }

    private static Map<String, String> getParmsMap(ModelInfo modelInfo) {
        String modelParms = modelInfo.getModelParms();
        if (ObjectUtil.isEmpty(modelParms)) {
            return new HashMap<>();
        }
        JSONArray modelParamsArray = new JSONArray(modelParms);
        Map<String, String> parmsMap = new HashMap<>();
        for (Object obj : modelParamsArray) {
            if (obj instanceof JSONObject) {
                JSONObject jsonObj = (JSONObject)obj;
                String name = jsonObj.getStr("key");
                String value = jsonObj.getStr("value");
                if (ObjectUtil.isNotEmpty(name) && ObjectUtil.isNotEmpty(value)) {
                    parmsMap.put(name, value);
                }
            }
        }
        return parmsMap;
    }

    private String getFileUrl(String dataParms, String dataPath) throws IOException {
        JSONArray jsonArray = new JSONArray(dataParms);
        // 查询数据源
        JSONArray dataJson = CsvUtils.getCsvDataByHeaders(dataPath, jsonArray.toList(String.class));
        // 生成新的csv文件
        String filePath =
            PHMConfig.getCsvFilePath() + CsvUtils.MODEL_DATA_PATH + CsvUtils.extractFilename("parameter_result");
        File file = CsvUtils.jsonToFileCsvByJsonArray(dataJson, filePath);
        // 上传并返回新文件名称以及路径
        String fileName = FileUploadUtils.upload(PHMConfig.getUploadPath(), FileUtil.fileToMultipartFile(file));
        log.info("上传的文件名称：{}", fileName);
        String url = Constants.HTTP + IpUtils.getIpAddr() + ":8080" + fileName;
        log.info("上传的文件地址：{}", url);
        return url;
    }
}
