package com.phm.manage.util;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import com.phm.common.utils.DateUtils;
import com.phm.common.utils.StringUtils;
import com.phm.common.utils.uuid.Seq;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.text.csv.CsvData;
import cn.hutool.core.text.csv.CsvReader;
import cn.hutool.core.text.csv.CsvRow;
import cn.hutool.core.text.csv.CsvUtil;
import cn.hutool.core.util.CharsetUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @Description CsvUtils
 * @Author WGK
 * @Date 2023/11/20 18:03
 */
@Slf4j
public class CsvUtils {

    // 模型需要的数据
    public static final String MODEL_DATA_PATH = "/modelData";

    // 执行模型后的结果数据
    public static final String RESULT_DATA_PATH = "/resultData/";

    // 源参数数据
    public static final String PARAMETER_DATA_PATH = "/parameterData/";
    public static final String CSV_TYPE = ".csv";

    /**
     * 编码文件名
     */
    public static final String extractFilename(String fileName) {
        return StringUtils.format("{}/{}_{}.{}", DateUtils.datePath(), fileName, Seq.getId(Seq.uploadSeqType),
            CSV_TYPE);
    }

    /**
     * json 转 csv数据
     *
     * @param jsonArray 数组
     * @return csv 结果
     */
    public static List<String> jsonArrayToCsv(JSONArray jsonArray) {
        // 获取CSV的表头
        List<String> headers = new ArrayList<>(jsonArray.getJSONObject(0).keySet());
        // 构造CSV行列表
        List<String> csvLines = jsonArray.stream().map(jsonObject -> {
            StringBuilder line = new StringBuilder();
            for (String header : headers) {
                // 根据表头获取对应的值，并追加到CSV行中
                line.append(((JSONObject)jsonObject).get(header)).append(",");
            }
            // 移除末尾的逗号
            return line.substring(0, line.length() - 1);
        }).collect(Collectors.toList());
        // 在CSV行列表开头添加表头
        csvLines.add(0, String.join(",", headers));
        return csvLines;
    }

    /**
     * json转csv文件
     *
     * @param jsonString data
     * @param csvFilePath path
     */
    public static File jsonToFileCsv(String jsonString, String csvFilePath) {
        try {
            // 将JSON字符串转换为JSON数组
            JSONArray jsonArray = JSONUtil.parseArray(jsonString);
            // 将JSON数组写入CSV文件
            return jsonToFileCsvByJsonArray(jsonArray, csvFilePath);
        } catch (Exception e) {
            log.error("json转csv文件错误：{}", e.getMessage());
        }
        return null;
    }

    /**
     * json转csv文件
     *
     * @param jsonArray data
     * @param csvFilePath path
     */
    public static File jsonToFileCsvByJsonArray(JSONArray jsonArray, String csvFilePath) {
        try {
            // 将JSON数组写入CSV文件
            return FileUtil.writeLines(jsonArrayToCsv(jsonArray), csvFilePath, CharsetUtil.CHARSET_GBK);
        } catch (Exception e) {
            log.error("json转csv文件错误：{}", e.getMessage());
        }
        return null;
    }

    /**
     * csv文件转Json
     *
     * @param csvFilePath path
     */
    public static JSONArray fileCsvToJson(String csvFilePath) {
        List<CsvRow> rows = getCsvRowList(csvFilePath);
        // 获取CSV表头，即第一行数据
        List<String> headers = rows.get(0).getRawList();
        // 去除第一行，保留后续数据
        List<CsvRow> dataLines = rows.subList(1, rows.size());
        JSONArray jsonArray = new JSONArray();
        for (CsvRow line : dataLines) {
            JSONObject jsonObject = new JSONObject();
            for (int i = 0; i < headers.size(); i++) {
                jsonObject.putOnce(headers.get(i), line.getRawList().get(i));
            }
            jsonArray.add(jsonObject);
        }
        return jsonArray;
    }

    /**
     * 获取csv文件中 参数名称
     *
     * @param csvFilePath csv文件地址
     * @return 参数名称列表
     */
    public static List<String> getCsvHeaders(String csvFilePath) {
        List<CsvRow> rows = getCsvRowList(csvFilePath);
        return rows.get(0).getRawList();
    }

    /**
     * 根据头名称(参数名称)获取csv文件中参数数据
     *
     * @param csvFilePath csv文件地址
     * @param headerNames 参数名称
     * @return 参数名称列表
     */
    public static JSONArray getCsvDataByHeaders(String csvFilePath, List<String> headerNames) {
        List<CsvRow> rows = getCsvRowList(csvFilePath);
        // 获取CSV表头，即第一行数据
        List<String> headers = rows.get(0).getRawList();
        List<Integer> indexList = new ArrayList<>();
        headerNames.forEach(name -> {
            int index = headers.indexOf(name);
            if (index != -1) {
                indexList.add(index);
            }
        });
        if (CollectionUtil.isEmpty(indexList)) {
            return null;
        }
        // 去除第一行，保留后续数据
        List<CsvRow> dataLines = rows.subList(1, rows.size());
        JSONArray jsonArray = new JSONArray();
        for (CsvRow line : dataLines) {
            JSONObject jsonObject = new JSONObject();
            for (int i = 0; i < headers.size(); i++) {
                // 参数索引位置
                if (indexList.contains(i)) {
                    jsonObject.putOnce(headers.get(i), line.getRawList().get(i));
                }
            }
            jsonArray.add(jsonObject);
        }
        return jsonArray;
    }

    private static List<CsvRow> getCsvRowList(String csvFilePath) {
        CsvReader reader = CsvUtil.getReader();
        // 从文件中读取CSV数据
        CsvData data = reader.read(FileUtil.file(csvFilePath), CharsetUtil.CHARSET_GBK);
        return data.getRows();
    }
}
