package com.phm.netty.domain;

import java.io.Serializable;
import java.util.Arrays;

import com.alibaba.fastjson2.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.phm.netty.enums.OrderEnum;
import com.phm.netty.utils.BitUtils;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import io.netty.util.CharsetUtil;
import lombok.Data;
import lombok.experimental.Accessors;

/**
 * @Description Message
 * @Author WGK
 * @Date 2023/9/14 11:41
 */
@Data
@Accessors(chain = true)
public class Message implements Serializable {
    /**
     * 数据长度
     */
    private int len;

    /**
     * 接收的通讯数据body
     */
    private String data;

    /**
     * 消息类型 0：参数传输 1：通用指令发送，具体指令类型有指令本身定义 2：指令应答 3：校时 4：心跳包
     */
    private short type;

    /**
     * 毫秒时间戳，从2020.1.1开始
     */
    private long timestamp = DateUtil.current() - DateUtil.parse("2020-01-01").getTime();

    /**
     * 数据源
     */
    private String source = "GPHM";

    /**
     * 目标
     */
    private String target = "SPHM";

    /**
     * 目标
     */
    private int reserver = 0;

    public Message() {}

    /**
     * 字节数组解析为 Message 对象
     *
     * @param msg Message 对象
     * @param buf 字节数组
     * @param len 要解析的长度
     */
    public static void bytesToMsg(Message msg, byte[] buf, int len) {
        int index = 0;
        msg.len = len;
        msg.type = BitUtils.toShort(buf, index);
        index += 2;
        msg.source = BitUtils.toString(buf, index);
        index += 8;
        msg.target = BitUtils.toString(buf, index);
        index += 8;
        msg.timestamp = BitUtils.toLong(buf, index);
        index += 8;
        msg.reserver = BitUtils.toInt(buf, index);
        index += 4;
        byte[] bytes = new byte[len - index];
        System.arraycopy(buf, index, bytes, 0, len - index);
        msg.data = new String(bytes, CharsetUtil.UTF_8);
    }

    public static void main(String[] args) {
        byte[] test1 = Message.testMsg();

        // 使用Arrays.toString()方法打印字节数组
        System.out.println("使用Arrays.toString()方法打印字节数组: " + Arrays.toString(test1));

        // 以十六进制形式打印字节数组
        System.out.print("以十六进制形式打印字节数组: ");
        for (byte b : test1) {
            System.out.print(String.format("%02X ", b));
        }
    }

    public static byte[] testMsg() {
        Message message = new Message();
        message.setType(OrderEnum.COMMON.getType()).setTarget("GPHM").setSource("SPHM")
            .setData("{\"id\": \"ZL001\",\"orderCode\":\"ZL001\"}");
        return Message.msgToBytes(message);
    }

    public static byte[] testMsg2() {
        Message message = new Message();
        message.setType(OrderEnum.RESPONSE.getType()).setTarget("GPHM").setSource("SPHM")
            .setData("{\"id\": \"ZL001\",\"response\":200}");
        return Message.msgToBytes(message);
    }

    public static byte[] testMsg3() {
        Message message = new Message();
        message.setType(OrderEnum.TIMING.getType()).setTarget("GPHM").setSource("SPHM")
            .setData("{\"id\": \"ZL001\",\"response\":200}");
        return Message.msgToBytes(message);
    }

    public static byte[] testMsg4() {
        Message message = new Message();
        message.setType(OrderEnum.ORDER_CONFIG.getType()).setTarget("GPHM").setSource("SPHM")
            .setData("{\"cmdId\": \"getConfig\"}");
        return Message.msgToBytes(message);
    }

    public static byte[] msgToBytes(Message msg) {
        if (ObjectUtil.isEmpty(msg)) {
            return new byte[0];
        }
        int dataLen = msg.data.getBytes().length;
        msg.setLen(dataLen + 30);
        byte[] bytes = new byte[msg.len + 4];
        byte[] msgLenBytes = BitUtils.getBytes(msg.len);
        System.arraycopy(msgLenBytes, 0, bytes, 0, 4);
        byte[] msgTypeBytes = BitUtils.getBytes(msg.type);
        System.arraycopy(msgTypeBytes, 0, bytes, 4, 2);
        byte[] msgSourceBytes = BitUtils.getBytes(msg.source, 8);
        System.arraycopy(msgSourceBytes, 0, bytes, 6, 8);
        byte[] msgTargetBytes = BitUtils.getBytes(msg.target, 8);
        System.arraycopy(msgTargetBytes, 0, bytes, 14, 8);
        byte[] msgTimeStampBytes = BitUtils.getBytes(msg.timestamp);
        System.arraycopy(msgTimeStampBytes, 0, bytes, 22, 8);
        byte[] msgReserveBytes = BitUtils.getBytes(msg.reserver);
        System.arraycopy(msgReserveBytes, 0, bytes, 30, 4);
        byte[] msgDataBytes = msg.getData().getBytes();
        System.arraycopy(msgDataBytes, 0, bytes, 34, msgDataBytes.length);
        return bytes;
    }

    public Message(Object object) {
        String str = object.toString();
        JSONObject jsonObject = JSONObject.parseObject(str);
        type = Short.parseShort(jsonObject.getString("type"));
        data = jsonObject.getString("data");
        len = str.length();
    }

    public static Message getMessage(byte[] binaryData) {
        // 处理接收到的二进制消息
        int len = 0;
        if (binaryData.length >= 4) {
            // 读取的字节长度
            len = BitUtils.toInt(binaryData, 0);
        }
        if (len <= 0) {
            return null;
        }
        // 创建一个新的字节数组来存储截取的部分
        byte[] subArray = new byte[len];
        // 方法将源字节数组的一部分复制到新数组中
        System.arraycopy(binaryData, 4, subArray, 0, len);
        Message msg = new Message();
        Message.bytesToMsg(msg, subArray, len);
        return msg;
    }

    public String toJsonString() {
        return "{" + "\n  \"len\": " + len + ",\n" + "  \"type\": " + type + ",\n" + "  \"source\": " + source + ",\n"
            + "  \"target\": " + target + ",\n" + "  \"timestamp\": " + timestamp + ",\n" + "  \"data\": " + data + "\n"
            + "}";
    }

    public String errorDate(String code) throws JsonProcessingException {
        ObjectMapper objectMapper = new ObjectMapper();
        return objectMapper.writeValueAsString(CommonResponse.buildSuccess(code));
    }

    public static String successDate(String code) throws JsonProcessingException {
        ObjectMapper objectMapper = new ObjectMapper();
        return objectMapper.writeValueAsString(CommonResponse.buildSuccess(code));
    }

    public static String success() throws JsonProcessingException {
        ObjectMapper objectMapper = new ObjectMapper();
        return objectMapper.writeValueAsString(CommonResponse.success());
    }
}
