package com.phm.netty.server.handler;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.phm.manage.domain.OrderConfig;
import com.phm.manage.domain.common.CommonResult;
import com.phm.netty.constant.MsgConstant;
import com.phm.netty.domain.Message;
import com.phm.netty.enums.OrderEnum;
import com.phm.netty.service.IProcessService;
import com.phm.netty.utils.ChannelMap;
import com.phm.manage.util.JsonUtils;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.util.AttributeKey;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.List;

/**
 * @Description ByteArrayMessageHandler
 * @Author WGK
 * @Date 2023/9/20 13:40
 */
@ChannelHandler.Sharable
@Slf4j
@Component
public class ByteArrayMessageHandler extends SimpleChannelInboundHandler<byte[]> {
    @Autowired
    IProcessService processService;

    private static ByteArrayMessageHandler byteArrayMessageHandler;

    @PostConstruct
    public void init() {
        byteArrayMessageHandler = this;
    }

    /**
     * 设备接入连接时处理
     *
     * @param ctx ctx
     */
    @Override
    public void handlerAdded(ChannelHandlerContext ctx) {
        String idStr = ctx.channel().id().asLongText();
        log.info("有新的连接：[{}]", idStr);
        log.info("校时请求");
        Message msg = new Message();
        msg.setType(OrderEnum.TIMING.getType()).setData("{\"cmdId\": \"timing\"}");
        ctx.writeAndFlush(Message.msgToBytes(msg));
    }

    /**
     * 客户端消息处理
     *
     * @param ctx        ctx
     * @param binaryData binaryData
     * @throws Exception Exception
     */
    @Override
    protected void channelRead0(ChannelHandlerContext ctx, byte[] binaryData) throws Exception {
        Message msg = Message.getMessage(binaryData);
        if (msg == null) {
            return;
        }
        short type = msg.getType();
        Message lastMsg = null;
        if (OrderEnum.COMMON.getType() == type && saveChannel(ctx, msg)) {
            // 通用指令
            lastMsg = byteArrayMessageHandler.processService.orderHandle(msg);
        }
        if (OrderEnum.TIMING.getType() == type) {
            // 校时功能
            byteArrayMessageHandler.processService.timing(msg);
        }
        if (OrderEnum.ORDER_CONFIG.getType() == type) {
            // 参数传递
            List<OrderConfig> configList = byteArrayMessageHandler.processService.getParameterConfig(msg);
            // 分批处理
            List<List<OrderConfig>> batches = CollUtil.split(configList, 10);
            Message msgParameter = new Message();
            for (List<OrderConfig> batch : batches) {
                msgParameter.setType(OrderEnum.ORDER_CONFIG.getType()).setData(JsonUtils.convertJson(CommonResult.success(batch)));
                log.info("msgParameter==={}", msgParameter);
                ctx.writeAndFlush(Message.msgToBytes(msgParameter));
            }
            // 结束传输标识
            msgParameter.setType(OrderEnum.ORDER_CONFIG.getType()).setData(JsonUtils.convertJson(CommonResult.complete("{\"cmdId\":\"complete\"}")));
            log.info("msgParameter end ==={}", msgParameter);
            ctx.writeAndFlush(Message.msgToBytes(msgParameter));
            return;
        }
        if (ObjectUtil.isEmpty(lastMsg)) {
            log.info("Message数据为空");
            return;
        }
        // 转byte[] 返回消息
        ctx.writeAndFlush(Message.msgToBytes(lastMsg));
    }

    /**
     * 保存消息通道 信息
     *
     * @param ctx ctx
     * @param msg msg
     * @return res
     * @throws JsonProcessingException ex
     */
    private boolean saveChannel(ChannelHandlerContext ctx, Message msg) throws JsonProcessingException {
        Message lastMsg = new Message();
        JSONObject jsonObject = JSONObject.parseObject(msg.getData());
        String code = jsonObject.getString("id");
        String orderCode = jsonObject.getString("orderCode");
        if (StringUtils.isBlank(code) || StringUtils.isBlank(orderCode)) {
            lastMsg.setType(OrderEnum.RESPONSE.getType()).setData(JsonUtils.convertJson(CommonResult.error("id and orderCode cannot be empty")));
            log.info("saveChannel lastMsg==={}", lastMsg);
            ctx.writeAndFlush(Message.msgToBytes(lastMsg));
            return false;
        }
        String idStr = MsgConstant.CLIENT_ID;
        if (ObjectUtil.isNotEmpty(ChannelMap.getChannel(idStr))) {
            log.info("已有消息通道");
            return true;
        }
        ChannelMap.getChannelMap().put(idStr, ctx.channel());
        // 将客户端ID作为自定义属性加入到channel中，方便随时channel中获取ID
        AttributeKey<String> key = AttributeKey.valueOf("id");
        ctx.channel().attr(key).setIfAbsent(idStr);
        return true;
    }

    /**
     * 设备下线处理
     *
     * @param ctx ctx
     */
    @Override
    public void handlerRemoved(ChannelHandlerContext ctx) {
        log.info("设备下线了:{}", ctx.channel().id().asLongText());
        // map中移除channel
        removeId(ctx);
    }

    /**
     * 设备连接异常处理
     *
     * @param ctx   ctx
     * @param cause cause
     */
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        // 打印异常
        log.error("异常：{}", cause.getMessage());
        // map中移除channel
        removeId(ctx);
        // 关闭连接
        ctx.close();
    }

    private void removeId(ChannelHandlerContext ctx) {
        AttributeKey<String> key = AttributeKey.valueOf("id");
        // 获取channel中id
        String id = ctx.channel().attr(key).get();
        // map移除channel
        if (CollectionUtil.isNotEmpty(ChannelMap.getChannelMap())) {
            log.info("删除channel: {}", id);
            ChannelMap.getChannelMap().remove(id);
        }
    }
}
